@@ -16,7 +16,7 @@ use CHI;
 use Locale::Country;
 
 our %admin1cache;
-our %admin2cache;
+our %admin2cache;	# e.g. maps 'Kent' => 'P5'
 
 # Some locations aren't found because of inconsistencies in the way things are stored - these are some values I know
 # FIXME: Should be in a configuration file
@@ -33,11 +33,11 @@ Geo::Coder::Free::Maxmind - Provides a geocoding functionality using the MaxMind
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =cut
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 SYNOPSIS
 
@@ -158,7 +158,7 @@ sub geocode {
 		if($location =~ /^St\.? (.+)/) {
 			$location = "Saint $1";
 		}
-		if(($country =~ /^(United States|USA|US)$/)) {
+		if(($country =~ /^(Canada|United States|USA|US)$/)) {
 			$state = $county;
 			$county = undef;
 		}
@@ -198,7 +198,7 @@ sub geocode {
 			$concatenated_codes = 'GB';
 		}
 		my $countrycode = country2code($country);
-	# 	::diag(__LINE__, ": country $countrycode, county $county, state $state, location $location");
+		# ::diag(__LINE__, ": country $countrycode, county $county, state $state, location $location");
 
 		if($state && $admin1cache{$state}) {
 			$concatenated_codes = $admin1cache{$state};
@@ -237,37 +237,48 @@ sub geocode {
 	# ::diag(__LINE__, ": $concatenated_codes");
 	return unless(defined($concatenated_codes));
 
-	$self->{'admin2'} //= Geo::Coder::Free::DB::MaxMind::admin2->new() or die "Can't open the admin2 database";
-
 	my @admin2s;
 	my $region;
 	my @regions;
+	# ::diag(__LINE__, ": $country");
 	if(($country =~ /^(United States|USA|US)$/) && $county && (length($county) > 2)) {
 		if(my $twoletterstate = Locale::US->new()->{state2code}{uc($county)}) {
 			$county = $twoletterstate;
 		}
-	} elsif(($country eq 'Canada') && (length($county) > 2)) {
-		if(my $twoletterstate = Locale::CA->new()->{province2code}{uc($county)}) {
-			$county = $twoletterstate;
+	} elsif(($country eq 'Canada') && (length($state) > 2)) {
+		# ::diag(__LINE__, ": $county");
+		if(my $twoletterstate = Locale::CA->new()->{province2code}{uc($state)}) {
+			# FIXME:  I can't see that province locations are stored in cities.csv
+			return unless(defined($location));	# OK if searching for a city, that works
+			# $state = $twoletterstate;
 		}
 	}
+
+	$self->{'admin2'} //= Geo::Coder::Free::DB::MaxMind::admin2->new() or die "Can't open the admin2 database";
+
 	if(defined($county) && ($county =~ /^[A-Z]{2}$/) && ($country =~ /^(United States|USA|US)$/)) {
 		# US state. Not Canadian province.
 		$region = $county;
 	} else {
 		if($county && $admin1cache{$county}) {
+		# ::diag(__LINE__);
 			$region = $admin1cache{$county};
 		} elsif($county && $admin2cache{$county}) {
+		# ::diag(__LINE__);
 			$region = $admin2cache{$county};
 		} elsif(defined($state) && $admin2cache{$state} && !defined($county)) {
+		# ::diag(__LINE__);
 			$region = $admin2cache{$state};
 		} else {
+		# ::diag(__LINE__);
 			if(defined($county) && ($county eq 'London')) {
 				@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $location);
 			} else {
+			# ::diag(__LINE__, ": $county");
 				@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $county);
 			}
 			foreach my $admin2(@admin2s) {
+				# ::diag(__LINE__, Data::Dumper->new([$admin2])->Dump());
 				if($admin2->{'concatenated_codes'} =~ $concatenated_codes) {
 					$region = $admin2->{'concatenated_codes'};
 					if($region =~ /^[A-Z]{2}\.([A-Z]{2})\./) {
@@ -307,7 +318,7 @@ sub geocode {
 	if((scalar(@regions) == 0) && !defined($region)) {
 		# e.g. Unitary authorities in the UK
 		# admin[12].db columns are labelled ['concatenated_codes', 'name', 'asciiname', 'geonameId']
-	# 	::diag(__LINE__, ": $location");
+	# 	# ::diag(__LINE__, ": $location");
 		@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $location);
 		if(scalar(@admin2s) && defined($admin2s[0]->{'concatenated_codes'})) {
 			foreach my $admin2(@admin2s) {
@@ -328,6 +339,7 @@ sub geocode {
 			}
 			my @admin1s = $self->{'admin1'}->selectall_hash(asciiname => $county);
 			foreach my $admin1(@admin1s) {
+				# ::diag(__LINE__, Data::Dumper->new([$admin1])->Dump());
 				if($admin1->{'concatenated_codes'} =~ /^$concatenated_codes\./i) {
 					$region = $admin1->{'concatenated_codes'};
 					$admin1cache{$county} = $region;
@@ -338,7 +350,8 @@ sub geocode {
 	}
 
 	if(!defined($self->{'cities'})) {
-		$self->{'cities'} = Geo::Coder::Free::DB::MaxMind::cities->new();
+		$self->{'cities'} = Geo::Coder::Free::DB::MaxMind::cities->new(
+			cache => $self->{cache} || CHI->new(driver => 'Memory', datastore => {}));
 	}
 
 	my $options;
@@ -368,6 +381,7 @@ sub geocode {
 		$options->{'Country'} = lc($c);
 		$confidence = 0.1;
 	}
+	# ::diag(__LINE__, Data::Dumper->new([$options])->Dump());
 	# This case nonsense is because DBD::CSV changes the columns to lowercase, wherease DBD::SQLite does not
 	if(wantarray) {
 		my @rc = $self->{'cities'}->selectall_hash($options);
@@ -441,29 +455,113 @@ sub geocode {
 
 	# ::diag(__LINE__, Data::Dumper->new([$city])->Dump());
 	if(defined($city) && defined($city->{'Latitude'})) {
-		$city->{'latitude'} = delete $city->{'Latitude'};
-		$city->{'longitude'} = delete $city->{'Longitude'};
 		$city->{'confidence'} = $confidence;
-		return Geo::Location::Point->new({
-			'lat' => $city->{'latitude'},
-			'long' => $city->{'longitude'},
-			'location' => $location
-		});
+		return Geo::Location::Point->new($city);
 	}
 	# return $city;
 	undef;
 }
 
-=head2 reverse_geocode
+=head2	reverse_geocode
 
     $location = $geocoder->reverse_geocode(latlng => '37.778907,-122.39732');
 
-To be done.
+Returns a string, or undef if it can't be found.
 
 =cut
 
 sub reverse_geocode {
-	Carp::croak('Reverse lookup is not yet supported');
+	my $self = shift;
+
+	my %param;
+	if(ref($_[0]) eq 'HASH') {
+		%param = %{$_[0]};
+	} elsif(ref($_[0])) {
+		Carp::croak('Usage: reverse_geocode(latlng => $location)');
+	} elsif(@_ % 2 == 0) {
+		%param = @_;
+	} else {
+		$param{'latlng'} = shift;
+	}
+
+	my $latlng = $param{'latlng'}
+		or Carp::croak('Usage: reverse_geocode(latlng => $location)');
+
+	my $latitude;
+	my $longitude;
+
+	if($latlng) {
+		($latitude, $longitude) = split(/,/, $latlng);
+	} else {
+		$latitude //= $param{'lat'};
+		$longitude //= $param{'lon'};
+		$longitude //= $param{'long'};
+	}
+
+	if(!defined($self->{'cities'})) {
+		$self->{'cities'} = Geo::Coder::Free::DB::MaxMind::cities->new(
+			cache => $self->{cache} || CHI->new(driver => 'Memory', datastore => {})
+		);
+	}
+
+	if(wantarray) {
+		my @locs = $self->{'cities'}->execute("SELECT * FROM cities WHERE (ABS(Latitude - $latitude) < 0.01) AND (ABS(Longitude - $longitude) < 0.01)");
+		# Change 'Charing Cross, P5, Gb' to 'Charing Cross, London, Gb'
+		foreach my $loc(@locs) {
+			if(my $region = $loc->{'Region'}) {
+				my $county;
+				foreach $county(keys %admin2cache) {
+					if($admin2cache{$county} eq $region) {
+						last;
+					}
+				}
+				if($county) {
+					$loc->{'Region'} = $county;
+				} else {
+					$self->{'admin2'} //= Geo::Coder::Free::DB::MaxMind::admin2->new() or die "Can't open the admin2 database";
+					my $row = $self->{'admin2'}->execute("SELECT name FROM admin2 WHERE concatenated_codes LIKE '" . uc($loc->{'Country'}) . '.%.' . uc($region) . "'");
+					if(ref($row) && $row->{'name'}) {
+						$admin2cache{$row->{'name'}} = $region;
+						$loc->{'Region'} = $row->{'name'};
+					}
+				}
+			}
+		}
+		# ::diag(Data::Dumper->new([\@locs])->Dump());
+		# my @rc;
+		# foreach my $loc(@locs) {
+			# push @rc, Geo::Location::Point->new($loc)->as_string();
+		# }
+		# return @rc;
+		return map { Geo::Location::Point->new($_)->as_string() } @locs;
+	}
+	# TODO - this is similar to the wantarray code, just the LIMIT 1 and
+	#	no map { } code.  Need to combine
+	if(my $rc = $self->{'cities'}->execute("SELECT * FROM cities WHERE (ABS(Latitude - $latitude) < 0.01) AND (ABS(Longitude - $longitude) < 0.01) LIMIT 1")) {
+		if(my $region = $rc->{'Region'}) {
+			my $county;
+			foreach $county(keys %admin2cache) {
+				if($admin2cache{$county} eq $region) {
+					last;
+				}
+			}
+			if($county) {
+				$rc->{'Region'} = $county;
+			} else {
+				$self->{'admin2'} //= Geo::Coder::Free::DB::MaxMind::admin2->new() or die "Can't open the admin2 database";
+				# ::diag("SELECT name FROM admin2 WHERE concatenated_codes LIKE '" . uc($rc->{'Country'}) . '.%.' . uc($region) . "' LIMIT 1");
+				my $row = $self->{'admin2'}->execute("SELECT name FROM admin2 WHERE concatenated_codes LIKE '" . uc($rc->{'Country'}) . '.%.' . uc($region) . "' LIMIT 1");
+				# print Data::Dumper->new([$row])->Dump();
+				# ::diag(Data::Dumper->new([$row])->Dump());
+				if(ref($row) && $row->{'name'}) {
+					$admin2cache{$row->{'name'}} = $region;
+					$rc->{'Region'} = $row->{'name'};
+				}
+			}
+		}
+		return Geo::Location::Point->new($rc)->as_string();
+	}
+	return;
 }
 
 =head2	ua
@@ -490,6 +588,8 @@ The MaxMind data only contains cities.
 
 Can't parse and handle "London, England".
 
+The database contains Canadian cities, but not provinces, so a search for "New Brunswick, Canada" won't work
+
 The GeoNames admin databases are in this class, they should be in Geo::Coder::GeoNames.
 
 =head1 SEE ALSO
